/*
 * asm.h
 * Rutinas para la ejecucion de codigo ensamblador para la arquitectura x86
 */

#ifndef ASM_H_
#define ASM_H_

#define inline_assembly(code...) \
		__asm__ __volatile__(code)

/*
 * Rutina: inb
 * Descripcion: Lee un byte de un puerto de entrada / salida.
 * Entrada:
 * 	unsigned short  : puerto de E/S del cual se debe leer el byte
 * Salida:
 *  unsigned char : byte leido del puerto.
 * */
static __inline__ unsigned char inb(unsigned short port) {
	unsigned char data;
	inline_assembly("inb %1,%0" : "=a" (data) : "dN" (port));
	return data;
}

/*
 * Rutina: outb
 * Descripcion: Escribe un byte en un puerto de entrada / salida.
 * Entrada:
 *  unsigned short : puerto de E/S al cual se puede escribir el byte
 *  unsigned char: : byte a escribir en el puerto.
 * Salida:
 * 	Ninguno.
 * */
static __inline__ void outb(unsigned short port, unsigned char data){
	inline_assembly("outb %1,%0" : : "dN" (port), "a" (data));
}


/*
 * Rutina: inw
 * Descripcion: Lee un word de un puerto de entrada / salida.
 * Entrada:
 * 	unsigned short  : puerto de E/S del cual se debe leer el word
 * Salida:
 *  unsigned short : word leido del puerto.
 * */
static __inline__ unsigned short inw(unsigned short port) {
	unsigned short data;
	inline_assembly("inw %1,%0" : "=a" (data) : "dN" (port));
	return data;
}

/*
 * Rutina: outw
 * Descripcion: Escribe un word en un puerto de entrada / salida.
 * Entrada:
 *  unsigned short : puerto de E/S al cual se puede escribir el word
 *  unsigned short: : word a escribir en el puerto.
 * Salida:
 * 	Ninguno.
 * */
static __inline__ void outw(unsigned short port, unsigned short data){
	inline_assembly("outw %1,%0" : : "dN" (port), "a" (data));
}

#endif /* ASM_H_ */
