/*
 * stdio.c
 * Este archivo contiene las primitivas basicas para entrada / salida
 */

#include <stdio.h>

/* Apuntador al inicio de la memoria de video */
unsigned short * videoptr = (unsigned short *) 0xB8000;

/* Variable que almacena los atributos de texto */
char text_attributes = 0x07;

/* Rutina: printchar
 * Descripcion: Rutina provisional para imprimir un caracter
 * Entrada:
 * 	char : caracter ascii a imprimir
 * Salida:
 *  Ninguno.
 * */
void putchar(char c) {
		/* Cada caracter que aparece en la pantalla corresponde a dos
		 * caracteres en la memoria de video: el byte menos significativo
		 * contiene el caracter ASCII a imprimir, y el byte mas significativo
		 * contiene los atributos de texto y fondo.
		 * Posteriormente se explicaran estos atributos. Por ahora se puede
		 * suponer que los caracteres apareceran en la pantalla en color
		 * gris claro con fondo negro. */

		*videoptr = (0x07 << 8 | c); /* Escribir el caracter en la memoria
									de video*/
		videoptr++; /* Avanzar el apuntador de la memoria de video */

		/* Verificar si se llego al final de la pantalla */
		if ( (unsigned int)videoptr >= 0xB8000 + (80 * 25) * 2) {
			videoptr = (unsigned short *) 0xB8000;
		}
}

/*
 * Rutina: puts
 * Descripcion: Rutina provisional para imprimir una cadena de caracteres.
 * Esta rutina no valida caracteres especiales.
 * Entrada:
 * 	char* : Cadena terminada en nulo que se desea imprimir
 * Salida:
 * 	Ninguno.
 * */
void puts(char * s ) {

	char * aux;
	char c;

	aux = s;

	if (aux == 0) {return;}

	while ((c = *aux++) != '\0'){
		putchar(c); /* simplemente imprimir el caracter*/
	}
}
