/* Codigo de inicializacion del kernel en C
 * Este codigo recibe el control de start.S y continua
 * con la ejecucion.
 *
 * En este kernel se implementan las rutinas necesarias para imprimir numeros
 * en diferente formato. Para ello, se implementan las rutinas que permiten
 * pasar un numero entero a string y un string a numero entero. Ver definiciones
 * en el archivo stdlib.h y las implementaciones en stdlib.c.
 *
 * En este proyecto se re-implementan las rutinas para imprimir un caracter
 * y una cadena de caracteres, de forma que actualicen el cursor del hardware
 * a la posicion actual de la memoria de video (ver rutina update_cursor en el
 * archivo stdio.c).
 * */

#include <multiboot.h>
#include <stdio.h>
#include <stdlib.h>
/*
 * Rutina: cmain
 * Descripcion: Rutina principal del kernel. Esta rutina recibe el control del
 * codigo en ensamblador de start.S.
 * Entrada:
 * 	int* : apuntador al numero de bytes que ocupa el kernel
 *  char*: mensaje de bienvenida.
 * */
void cmain(unsigned int magic, void * multiboot_info) {

	/* Buffer para itoa */
	char linea[80];

	/* Numero para almacenar el string transformado a entero*/
	int n;

	/* Limpiar la pantalla */
	cls();

	/* Mensaje de bienvenida */
	puts("Hello from C kernel! ");

	/* Transformar el numero magico en string e imprimirlo */
	puts("Magic number:  ");

	itoa(magic, linea, 16); /* linea = magic transformado a string */
	puts(linea); /* Imprimir linea */

	putchar('='); /* Probar putchar */

	/* Transformar la linea en un numero, luego transformar el numero
	 * en string. Deberia imprimir el mismo numero magico. */

	n = atoi(linea, 16);
	itoa(n, linea, 16);
	puts(linea);

	/* Probar atoi con otra cadena, base 10 */
	puts("\nDecimal:\n");
	n = atoi(" -123 ", 10); /* Convertir a numero en base 10 */
	itoa(n, linea, 10); /* Convertir el numero a cadena, en base 10 */
	puts(linea); /* Deberia imprimir el mismo numero */

	/* Probar atoi con otra cadena, base 10 */
	puts("\n\tBinario:\n");
	n = atoi(" 1010 ", 2); /* Convertir a numero en base 2 */
	itoa(n, linea, 2); /* Convertir el numero a cadena, en base 2 */
	puts(linea); /* Deberia imprimir el mismo numero */
	puts(":::");

	/* Probar atoi con otra cadena, base 16 */
	puts("\n\tHexa: ");
	n = atoi(" 7c00 ", 16); /* Convertir a numero en base 16 */
	itoa(n, linea, 16); /* Convertir el numero a cadena, en base 16 */
	puts(linea); /* Deberia imprimir el mismo numero */

	/* Imprimir la posicion actual de la memoria de video */
	puts(" Posicion actual de video : ");
	itoa((unsigned int)video_ptr, linea, 16); /* Convertir el numero a cadena, en base 16 */
	puts(linea); /* Deberia imprimir el mismo numero */

	/* Imprimir los atributos de texto, definidos en stdio.h */
	puts(" Atributos de texto: ");
	itoa(text_attributes, linea, 16); /* Convertir el numero a cadena, en base 16 */
	puts(linea); /* Deberia imprimir el mismo numero */
}
