/*
 * stdio.h
 * Este archivo contiene las primitivas basicas para entrada / salida
 */

#ifndef STDIO_H_
#define STDIO_H_

#include <asm.h> /* Para operaciones sobre puertos de E/S */

#define VIDEO_ADDR 0XB8000

/* Apuntador al inicio de la memoria de video. Definido en stdio.c */
extern unsigned short * video_ptr;

/* Variable que almacena los atributos de texto. Definida en stdio.c */
extern char text_attributes;

/* Valores por defecto para la pantalla VGA de modo texto */

#define SCREEN_LINES 25 /* Numero de lineas de la pantalla */
#define SCREEN_COLUMNS 80 /* Numero de columnas de la pantalla */

#define TABSIZE 8 /* Espacios en un tabulador */

/* Variable que controla el numero de lineas de la pantalla */
extern int screen_lines;

/* Variable que controla el numero de columnas de la pantalla */
extern int screen_columns;

/* Macro para calcular el byte de atributos de texto y fondo */
#define COLOR(fg, bg) ((bg << 4) | fg)

/* Constantes de colores de 4 bits */
#define BLACK 0
#define BLUE 1
#define GREEN 2
#define CYAN 3
#define RED 4
#define MAGENTA 5
#define BROWN 6
#define LIGHTGRAY 7
#define DARKGRAY 8
#define LIGHTBLUE 9
#define LIGHTGREEN 10
#define LIGHTCYAN 11
#define LIGHTRED 12
#define LIGHTMAGENTA 13
#define LIGHTBROWN 14
#define WHITE 15

/* Algunos caracteres especiales */
#define SPACE 0x20
#define BACKSPACE 0x08
#define LF '\n'
#define CR '\r'
#define TAB 0x09

/* Rutina: putchar
 * Descripcion: Rutina  para imprimir un caracter.
 * Esta rutina valida caracteres especiales, como \n, \t o \b.
 * Entrada:
 * 	char : caracter ascii a imprimir
 * Salida:
 *  Ninguno.
 * */
void putchar(char c);

/*
 * Rutina: puts
 * Descripcion: Rutina para imprimir una cadena de caracteres.
 * Esta rutina valida caracteres especiales, como \n, \t o \b.
 * Entrada:
 * 	char* : Cadena terminada en nulo que se desea imprimir
 * Salida:
 * 	Ninguno.
 * */
void puts(char * s );

/*
 * Rutina: cls
 * Descripcion: Rutina para limpiar la pantalla
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.
 * */
void cls(void);


/*
 * Rutina: printf
 * Descripcion:  Esa funcion implementa en forma basica el comportamiento de
 * 'printf' en C.
 * Entrada: format: Formato de la cadena de salida
 *          ... : Lista de referencias a memoria de las variables a imprimir
 *
*/
void printf(char * ,...);

#endif /* STDIO_H_ */
