/* Codigo de inicializacion del kernel en C
 * Este codigo recibe el control de start.S y continua
 * con la ejecucion.
 *
 * En este proyecto se implementa la rutina printf (ver archivo stdio.c), la
 * cual permite caracteres, numeros en diferente base y cadenas de caracteres.
 * */

#include <multiboot.h>
#include <stdio.h>
#include <stdlib.h>

/*
 * Rutina: cmain
 * Descripcion: Rutina principal del kernel. Esta rutina recibe el control del
 * codigo en ensamblador de start.S.
 * 	int* : apuntador al numero de bytes que ocupa el kernel
 *  char*: mensaje de bienvenida.
 * */
void cmain(unsigned int magic, void * multiboot_info) {

	int n; /* Numero entero */

	cls();

	/* Probar la implementacion de la rutina printf. */
	printf("Hello from protected mode!\n");

	/* Imprimir el numero magico */
	printf("\tMagic number: 0x%x ", magic);

	/* Imprimir la direccion de memoria en la cual se encuentra la
	 * estructura de datos con la informacion que GRUB le paso al
	 * kernel */
	printf("Location of the multiboot information structure: 0x%x\n",
			(unsigned int)multiboot_info);

	/* Imprimir la direccion de memoria en la cual GRUB copio esta rutina */
	printf("Location of this routine: 0x%x\n", &cmain);

	printf("Printing some numbers in several bases\n");

	/* Imprimir la direccion de inicio de la memoria de video */
	n = 0xB8000;
	printf("dec: %d hex: %x bin: %b\n", n, n, n);

	/* Imprimir 2^31 en diferentes formatos  */

	n = 0x80000000;
	printf("binary: %b signed: %d unsigned:%u hex: %x\n", n, n, n, n);

	/* Imprimir un numero negativo en diferentes formatos */
	n = -1;
	printf("binary: %b signed: %d unsigned:%u hex: %x\n", n, n, n, n);
}
