/* Codigo de inicializacion del kernel en C
 * Este codigo recibe el control de start.S y continua
 * con la ejecucion.
 *
 * Este proyecto implementa las primitivas necesarias para gestionar la memoria.
 * Para ello hace uso de la memoria disponible de la siguiente forma:
 * 1. La memoria por debajo de 1 MB se reserva para las estructuras de datos
 * del kernel.
 * 2. La memoria por encima del kernel y hasta el limite de memoria disponible
 * se reserva y libera automaticamente.
 *
 * Las rutinas para gestionar memoria se definen en el archivo include/mm.h
 * y se implementan en el archivo src/mm.c
 *
 * */

#include <multiboot.h>
#include <stdio.h>
#include <stdlib.h>
#include <mm.h>

/*
 * Rutina: cmain
 * Descripcion: Rutina principal del kernel. Esta rutina recibe el control del
 * codigo en ensamblador de start.S.
 * 	int* : apuntador al numero de bytes que ocupa el kernel
 *  char*: mensaje de bienvenida.
 * */


void cmain(unsigned int magic, unsigned int multiboot_info) {

	/* Variables para gestionar la memoria disponible */
	unsigned int memory_start; /* Inicio de la memoria disponible */
	unsigned int memory_length; /* Tamano de la memoria disponible */

	/* Recuperar los valores del encabezado multiboot definidos en el
		 * archivo start.S */
	extern multiboot_header_t multiboot_header;

	/* Limpiar la pantalla */
	cls();

	char * linea;

	/* Apuntar a la estructura de datos de informacion pasada por GRUB */
	multiboot_info_t *info = (multiboot_info_t *)multiboot_info;

	/* Buscar la region de memoria disponible, que se encuentra
		 * por encima del kernel*/
	get_available_memory(multiboot_header, info, &memory_start, &memory_length);

	/* Se encontro memoria disponible? */
	if (memory_length > 0) {
		printf("Available memory found at %x. Length: %dK\n",
				memory_start, memory_length / 1024);
	}else {
		printf("No available memory found!\n");
	}

	/* Inicializar el heap del kernel, con la informacion que se recibe
	 * de GRUB */
	mm_init((void *)memory_start, memory_length);

	printf("Allocated pointer at: %u\n", (unsigned int)mmalloc(20));
	printf("Allocated pointer at: %u\n", (unsigned int)mmalloc(20));
	printf("Allocated pointer at: %u\n", (unsigned int)mmalloc(20));

	printf("Allocated pointer at: %u\n", (unsigned int)mmalloc(320));
	printf("Allocated pointer at: %u\n", (unsigned int)mmalloc(320));
	printf("Allocated pointer at: %u\n", (unsigned int)mmalloc(320));
	printf("Allocated pointer at: %u\n", (unsigned int)mmalloc(5));

	printf("Test finished\n");

}
