/*
 * interrupt.h
 * Este archivo define las rutinas publicas para la gestion de solicitudes
 * de interrupcion (IRQ) de los dispositivos de Entrada / Salida
 */

#ifndef INTERRUPT_H_
#define INTERRUPT_H_

/* Desplazamiento en la IDT a partir de la cual se configuran las rutinas
 *  de manejo de interrupcion */
#define IDT_INTERRUPT_OFFSET 32

/* Ahora definir las constantes para los numeros de interrupcion de
 * las IRQ0 - IRQ15.*/

#define IRQ0_INTERRUPT IDT_INTERRUPT_OFFSET + 0
#define IRQ1_INTERRUPT IDT_INTERRUPT_OFFSET + 1
#define IRQ2_INTERRUPT IDT_INTERRUPT_OFFSET + 2
#define IRQ3_INTERRUPT IDT_INTERRUPT_OFFSET + 3
#define IRQ4_INTERRUPT IDT_INTERRUPT_OFFSET + 4
#define IRQ5_INTERRUPT IDT_INTERRUPT_OFFSET + 5
#define IRQ6_INTERRUPT IDT_INTERRUPT_OFFSET + 6
#define IRQ7_INTERRUPT IDT_INTERRUPT_OFFSET + 7
#define IRQ8_INTERRUPT IDT_INTERRUPT_OFFSET + 8
#define IRQ9_INTERRUPT IDT_INTERRUPT_OFFSET + 9
#define IRQ10_INTERRUPT IDT_INTERRUPT_OFFSET + 10
#define IRQ11_INTERRUPT IDT_INTERRUPT_OFFSET + 11
#define IRQ12_INTERRUPT IDT_INTERRUPT_OFFSET + 12
#define IRQ13_INTERRUPT IDT_INTERRUPT_OFFSET + 13
#define IRQ14_INTERRUPT IDT_INTERRUPT_OFFSET + 14
#define IRQ15_INTERRUPT IDT_INTERRUPT_OFFSET + 15


/* Esta constante define el numero maximo de rutinas de manejo de interrupcion
 * que se pueden definir en el sistema.
 * Inicialmente solo se definen 16 interrupciones, que corresponden a
 * las interrupciones que pueden generar los dispositivos de E/S (15) + 1.*/
#define MAX_INTERRUPT_ROUTINES 16


/*
 * Rutina: setup_interrupts
 * Descripcion: Esta rutina se encarga de crear las entradas en la IDT para
 * las interrupciones que se desean manejar. Por defecto configura las
 * interrupciones correspondientes a las IRQ de los dispositivos de entradda/
 * salida, que han sido mapeadas a los numeros 32..47.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.*/
void setup_interrupts(void);

/*
 * Rutina: install_interrupt_handler
 * Descripcion: Esta rutina permite definir un nuevo manejador de IRQ dentro
 * de la tabla de descriptores de interrupcion (IDT)
 * Entrada:
 * 	int: numero de interrupcion a configurar
 *  void (*)(struct interrupt_state) : Rutina a manejar la interrupcion
 * Salida:
 * 	Ninguno. */
void install_interrupt_handler(int, void (*)(struct interrupt_state));

/*
 * Rutina: uninstall_interrupt_handler
 * Descripcion: Esta rutina permite quitar un  manejador de IRQ dentro
 * de la tabla de descriptores de interrupcion (IDT)
 * Entrada:
 * 	int: numero de interrupcion a configurar
 *  void (*)(struct interrupt_state) : Rutina a manejar la interrupcion
 * Salida:
 * 	Ninguno. */
void uninstall_interrupt_handler(int);

#endif /* INTERRUPT_H_ */
