/*
 * task.c
 * Este archivo implementa las rutinas asociadas con la gestion de TSS.
 */

#include <pm.h>
#include <asm.h>
#include <stdlib.h>
#include <tss.h>

/* Definicion del segmento de estado de tarea (TSS) para el kernel.*/
tss kernel_tss __attribute__((aligned(8)));

/* Selector del TSS del kernel */
unsigned short kernel_tss_selector;

/*
 * Rutina: setup_tss
 * Descripcion: Esta rutina establece el registro de estado de tarea (TSS) para
 * el kernel.
 * Entrada:
 * 	Ninguno.
 * Salida:
 *  Ninguno.
 * */
void setup_tss(void) {

	unsigned int current_esp;

	/* Variable para almacenar el indice dentro del GDT que se asignara al
	 * descriptor de TSS*/
	unsigned short tss_index;

	/* Inicializar la estructura de datos Task State Segment del kernel.*/
	kernel_tss = NULL_TSS;

	/* Buscar una entrada en la GDT para el TSS */
	tss_index = allocate_gdt_entry();

	unsigned int base = (unsigned int)&kernel_tss;
	unsigned int limit = sizeof(tss) - 1;

	//printf("TSS index: %d\n", tss_index);

	/* Configurar la entrada en la GDT para el descriptor TSS y obtener el
	 * selector */
	kernel_tss_selector = setup_gdt_descriptor(tss_index,
			tss_descriptor_32(&kernel_tss, RING0_DPL));

	/* Imprimir el descriptor creado*/
	//printf("GDT descriptor before task switch..\n");
	//print_gdt_descriptor(tss_index);

	//printf(" Loading LTR register ...");

	inline_assembly("ltr %0"::"g"(kernel_tss_selector));

	//printf(" Done! ");

}

