/* start.S. Punto de entrada del kernel
* El kernel se carga en la direccion de memoria 0x100000 (1 MB) y empieza su
* ejecucion en la etiqueta start.
* Este kernel cumple con la especificacion multiboot, por lo cual debe
* contener un encabezado (estructura de datos) que le indica al cargador
* de arranque los parametros necesarios para que cargue el kernel.
* Este kernel es diferente a los anteriores: Se almacena en una imagen de disco
* duro, que se encuentra formateada con el sistema de archivos ext2 (linux).
* Para cargarlo, se recurre a  un cargador de arranque (bootloader) llamado
  GRUB.
* GRUB es un bootloader que cumple con la Especificacion Multiboot. Puede
* cargar cualquier kernel que cumpla con esta especificacion.
*/

.intel_syntax noprefix /* Usar sintaxis Intel, si prefijo para los registros */
/* Seccion llamada 'boot' para almacenarla al principio del archivo ejecutable
de salida (Ver archivo link.ld). La especificacion Multiboot exige que este
encabezado se encuentre en los primeros 8192 bytes del archivo ejecutable. */
.section .boot
.code32				/* 32 bits - Modo protegido */

.globl start	/* Punto de entrada del kernel */

#define ASM 1 /* Solo incluir las constantes del archivo multiboot.h*/

#include <multiboot.h>

start:
  jmp entry_point

.align 4
/* Exportar este simbolo para que este disponible en C. */
.globl multiboot_header
multiboot_header:		/* Encabezado multiboot */
.long MULTIBOOT_HEADER_MAGIC
.long MULTIBOOT_HEADER_FLAGS
.long MULTIBOOT_CHECKSUM

.long multiboot_header /* Header address Direccion en la cual se encuentra
						  el encabezado multiboot */

/* kernel_start, data_end, kernel_end son calculados por el linker. Ver
   archivo link.ld*/

.long kernel_start  /* Load address  = la direccion fisica del comienzo
					   del segmento de texto */
.long data_end   /* Load end address = la direccion fisica del final del
					   segmento de datos  */
.long kernel_end   /* BSS end address = Direccion fisica del final del segmento
						BSS. El cargador inicializa esta area a cero */
.long entry_point /* Entry address  = Direccion fisica a la cual el cargador
					deberia saltar para comenzar la ejecucion del sistema
					operativo */

entry_point: /* Punto de inicio de la ejecucion del kernel*/

  cli

  mov esp, 0x9FC00	/* Tope de la pila en 0x9FC00*/

  /* Reset EFLAGS*/
  push 0
  popf

  /* Enviar los parametros al kernel */
  push %ebx /* Direccion fisica de memoria en la cual se encuentra
  			   la estructura de informacion multiboot. Esta puede
  			   ser utilizada por el kernel para obtener la informacion
  			   suministrada por el cargador. */
  push %eax /* Numero magico del cargador de arranque = 0x2BADB002.
  			   En el kernel se puede validar si se recibio este valor. En caso
  			   afirmativo, el kernel fue cargado por un cargador que cumple
  			   con la especificacion multiboot. */

  call cmain /* Pasar el control a la rutina 'cmain' en elarchivo kernel.c */

  add sp, 8

loop:	hlt
	jmp loop

