/*
 * irq.h
 * Este archivo define las rutinas publicas para la gestion de solicitudes
 * de interrupcion (IRQ) de los dispositivos de Entrada / Salida
 */

#ifndef IRQ_H_
#define IRQ_H_

/* Desplazamiento en la IDT a partir de la cual se configuran las rutinas
 *  de manejo de interrupcion */
#define IDT_IRQ_OFFSET 32


/* Alias para el manejador de irq. */
typedef interrupt_handler irq_handler;

/* Ahora definir las constantes para los numeros de interrupcion de
 * las IRQ0 - IRQ15.*/

#define IRQ0_INTERRUPT IDT_IRQ_OFFSET + 0
#define IRQ1_INTERRUPT IDT_IRQ_OFFSET + 1
#define IRQ2_INTERRUPT IDT_IRQ_OFFSET + 2
#define IRQ3_INTERRUPT IDT_IRQ_OFFSET + 3
#define IRQ4_INTERRUPT IDT_IRQ_OFFSET + 4
#define IRQ5_INTERRUPT IDT_IRQ_OFFSET + 5
#define IRQ6_INTERRUPT IDT_IRQ_OFFSET + 6
#define IRQ7_INTERRUPT IDT_IRQ_OFFSET + 7
#define IRQ8_INTERRUPT IDT_IRQ_OFFSET + 8
#define IRQ9_INTERRUPT IDT_IRQ_OFFSET + 9
#define IRQ10_INTERRUPT IDT_IRQ_OFFSET + 10
#define IRQ11_INTERRUPT IDT_IRQ_OFFSET + 11
#define IRQ12_INTERRUPT IDT_IRQ_OFFSET + 12
#define IRQ13_INTERRUPT IDT_IRQ_OFFSET + 13
#define IRQ14_INTERRUPT IDT_IRQ_OFFSET + 14
#define IRQ15_INTERRUPT IDT_IRQ_OFFSET + 15

/* Esta constante define el numero maximo de rutinas de manejo de IRQ
 * que se pueden definir en el sistema.*/
#define MAX_IRQ_ROUTINES 16

/*
 * Rutina: setup_irq
 * Descripcion: Esta rutina se encarga de crear las entradas en la IDT para
 * las interrupciones que se desean manejar. Por defecto configura las
 * interrupciones correspondientes a las IRQ de los dispositivos de entradda/
 * salida, que han sido mapeadas a los numeros 32..47.
 * Entrada:
 * 	Ninguno.
 * Salida:
 * 	Ninguno.*/
void setup_irq(void);

/*
 * Rutina: install_irq_handler
 * Descripcion: Esta rutina permite definir un nuevo manejador de IRQ
 * Entrada:
 * 	int: numero de irq a configurar
 *  irq_handler : Rutina a manejar la irq
 * Salida:
 * 	Ninguno. */
void install_irq_handler(int, irq_handler);

/*
 * Rutina: uninstall_irq_handler
 * Descripcion: Esta rutina permite quitar un  manejador de IRQ.
 * Entrada:
 * 	int: numero de irq a quitar
 * Salida:
 * 	Ninguno. */
void uninstall_irq_handler(int);

#endif /* IRQ_H_ */
