/*
 * stdlib.h
 * Este archivo contiene las definiciones de algunas funciones de utilidad.
 */

#ifndef STDLIB_H_
#define STDLIB_H_

/* Tipo de datos void * */
typedef void * type_t;

/* Tipo de datos para apuntador char * */
typedef char * ptr_t;

/* Macro para verificar el valor de un bit especifico de una variable */
#define test_bit(x, n) (x & (1 << n))

/* Macro para establecer en 1 el valor de un bit de la variable */
#define set_bit(x, n) x |= (1 << n)

/* Macro para establecer en 0 el valor de un bit de la variable */
#define clear_bit(x, n)  x &= ~(1<<n)

/*
 * Rutina: itoa
 * Descripcion: Convierte un numero en base 2, 10 0 16 a un string terminado
 * en nulo. Si la base es 10, toma el numero con signo.
 * Entrada:
 * 	char * : Buffer en el cual se almacenara el numero como string
 *  int: Base en la cual se encuentra el numero
 * Salida:
 *  char * : Buffer en el cual se almacena el numero como string
 * */
char * itoa(unsigned int, char *, int);

/*
 * Rutina: utoa
 * Descripcion: Convierte un numero en base 2, 10 0 16 a un string terminado
 * en nulo. El numero se toma como entero sin signo.
 * Entrada:
 * 	char * : Buffer en el cual se almacenara el numero como string
 *  int: Base en la cual se encuentra el numero
 * Salida:
 *  char * : Buffer en el cual se almacena el numero como string
 * */
char * utoa(unsigned int, char *, int);

/* Rutina: atoi
 * Descripcion: Convierte un string a un entero, en la base especificada
 * Entrada:
 * 	char * : Buffer que contiene el numero
 * 	int : Base en la cual se quiere transformar el numero
 * Salida:
 *  int: Numero en la base especificada.
 * */
int atoi(char *, int );




#endif /* STDLIB_H_ */
