/*
 * string.h
 * Este archivo contiene algunas definiciones para funciones sobre
 * cadenas de caracteres y regiones de memoria.
 */

#ifndef STRING_H_
#define STRING_H_

/*
 * Rutina: memcpy
 * Descripcion: Esta rutina permite copiar un numero determinado de bytes
 * de una posicion de memoria a otra.
 * Entrada:
 * 	void * : Direccion de memoria de destino de los datos
 *  void * : Direccion de memoria de fuente de los datos
 *  int	   : Numero de bytes a copiar
 * Salida:
 *  void * : Direccion de memoria de destino de los datos
 */
void *memcpy(void *, const void *, int );

/*
 * Rutina: memset
 * Descripcion: Copia auna region de memoria un valor (char) determinado.
 * Entrada:
 * 	void *  : Direccion de memoria de destino
 *  char :  Valor a copiar
 *  int	: Numero de repeticiones
 * Salida :
 * 	void * : Direccion de memoria de destino
 *   */
void *memset(void *, char , int );

/*
 * Rutina: strlen
 * Descripcion: Esta rutina permite determinar la longitud de una cadena
 * terminada en el caracter nulo.
 * Entrada:
 * 	char *  : Apuntador al inicio de la cadena de caracteres
 * Salida:
 *  int	: Longitud de la cadena.
 *  */
int strlen(const char *);

/*
 * Rutina: strcpy
 * Descripcion: Esta rutina permite copiar una cadena a otra.
 * Entrada:
 * 	char * : Apuntador a la cadena destino
 *  char * : Apuntador a la cadena fuente
 * Salida:
 *  int	 : Numero de caracteres copiados.
 */
int strcpy(char *, const char *);

/*
 * Rutina: strcmp
 * Descripcion: Esta rutina permite comparar dos cadenas de caracteres.
 * Entrada:
 * 	char * : Apuntador a la cadena de destinno
 *  char * : Apuntador a la cadena fuente
 * Salida:
 *  int	 : Resultado de la comparacion ( 0 :  cadenas iguales,
 *  		!= 0 : cadenas diferentes)
 */
int strcmp(const char *, const char *);


#endif /* STRING_H_ */
