/*
 * string.c
 * Este archivo implementa algunas rutinas para la gestion de direcciones
 * de memoria y cadenas de caracteres.
 */

#include <string.h>

/*
 * Rutina: memcpy
 * Descripcion: Esta rutina permite copiar un numero determinado de bytes
 * de una posicion de memoria a otra.
 * Entrada:
 * 	void * : Direccion de memoria de destino de los datos
 *  void * : Direccion de memoria de fuente de los datos
 *  int	   : Numero de bytes a copiar
 * Salida:
 *  void * : Direccion de memoria de destino de los datos
 */
void *memcpy(void *dst, const void *src, int count) {
	const char *sp = (const char *) src;
	char *dp = (char *)dst;
	for (;count != 0; count--) {
		*dp = *sp;
		dp++;
		sp++;
	}
	return dst;
}

/*
 * Rutina: memset
 * Descripcion: Copia auna region de memoria un valor (char) determinado.
 * Entrada:
 * 	void *  : Direccion de memoria de destino
 *  char :  Valor a copiar
 *  int	: Numero de repeticiones
 * Salida :
 * 	void * : Direccion de memoria de destino
 *   */
void *memset(void *dst, char val, int count) {
	char *tmp = (char *) dst;
	for (;count !=0; count--) {
		*tmp = val;
		tmp++;
	}
	return dst;
}

/*
 * Rutina: strlen
 * Descripcion: Esta rutina permite determinar la longitud de una cadena
 * terminada en el caracter nulo.
 * Entrada:
 * 	char *  : Apuntador al inicio de la cadena de caracteres
 * Salida:
 *  int	: Longitud de la cadena.
 *  */
int strlen(const char *str){
	int ret;
	for (ret = 0; *str != '\0'; str++, ret++);
	return ret;
}

/*
 * Rutina: strcpy
 * Descripcion: Esta rutina permite copiar una cadena a otra.
 * Entrada:
 * 	char * : Apuntador a la cadena destino
 *  void * : Apuntador a la cadena fuente
 * Salida:
 *  int	 : Numero de caracteres copiados.
 */
int strcpy(char *dst, const char *src) {
	const char * x;
	char *y;
	int count;

	for (x=src, y=dst, count=0; *x != 0; y++,x++, count++){
		*y = *x;
	}
	*y=0; //Finalizar la segunda cadena
	return count;
}
/*
 * Rutina: strcmp
 * Descripcion: Esta rutina permite comparar dos cadenas de caracteres.
 * Entrada:
 * 	char * : Apuntador a la cadena de destinno
 *  char * : Apuntador a la cadena fuente
 * Salida:
 *  int	 : Resultado de la comparacion ( 0 :  cadenas iguales,
 *  		!= 0 : cadenas diferentes)
 */
int strcmp(const char *a, const char *b) {
	const char *x = a;
	const char *y = b;

	/* Avanzar hasta encontrar fin de alguna cadena o caracter diferente */
	for (x = a, y = b; *x != 0 && *y != 0 && x == y; x++, y++);

	/* Verificar si una cadena es mas corta que la otra*/
	if (*x == 0) {
		if (*y == 0) {
			return 0;
		}
		return -1;
	}else {
		if (*y == 0) {
			return 1;
		}
		return *y - *x;
	}
}


/*
 * Rutina: nexttok
 * Descripcion: Permite obtener una 'palabra' de una cadena de entrada.
 * Entrada:
 *  char * : Cadena de entrada
 *  char * : Cadena en la cual se almacena la palabra obtenida
 *  int: Desplazamiento inicial en la cadena de entrada
 * Salida:
 *  int: Numero de caracteres consumidos de la cadena de entrada
 * */
int nexttok(char * source, char * destination, int offset) {
    char * src =  source;
    char * dst = destination;
    char c;
    int nchars;

    //printf("%s", source);
    c=0;
    nchars = 0;

    if (src == 0) {
            return 0;
    }
    if (*src == 0) {
             return 0;
    }

    while ( (c=src[offset++]) !=0 && c == ' '){ //Ignorar espacios antes
       nchars++;
    }


    if (c == 0) { //La cadena de entrada se termino!
          return 0;
    }else { //Regresar un caracter!
          offset--;
    }

    //printf("Token start: %d %c nchars: %d\n", offset, src[offset], nchars);

    while ( (c=src[offset++]) != 0 && c != ' ') { //Sacar el valor
          *dst++ = c;
          nchars++;
    }
    //printf("Chars read: %d\n", nchars);

    if (nchars > 0) { //Terminar correctamente la cadena de destino
      *dst = 0;
    }

    //printf("Resulting token: %s bytes read: %d\n", destination, nchars);

   return nchars;
}

