/*
 * start.S
 * Este archivo implementa el entorno de inicializacion antes de saltar al
 * codigo principal de la tarea.
 * Al finalizar la rutina main() de la tarea, se retorna al codigo de este
 * archivo. Con el fin de informar al sistema operativo que la tarea ha
 * finalizado, se emite la interrupcion que fue programada para que pudiera
 * ser lanzada por las tareas (0x80).
 *
 */

.section .bootcode
.code32
/* Rutina: start
* Descripcion: Esta rutina es la primera a ejecutarse cuando la tarea recibe
* el control del kernel. Se encarga de invocar a la rutina main() del codigo
* de la tarea.
*/
.global start
start:
	call main
	/* Por convencion, el valor de retorno de una rutina se almacena en el
	 registro %eax.*/
	/* Saltar a la rutina task_finished, que se encarga de informar al kernel
	que la tarea termino por medio de una interrupcion. */
	jmp finished

finished:
	/* %eax contiene el valor de retorno de la rutina main(). */
	push %eax	/* Segundo argumento al manejador de tareas: Valor de retorno
				de main */

	push $0x01	/* Primer argumento al manejador de tareas */

	/* Invocar a la rutina kernel_exit, que se encarga de avisar al kernel
	que la tarea termino por medio de una interrupcion. */
	call kernel_exit

	/* El kernel no deberia retornar a la tarea. No obstante, entrar a un ciclo
	infinito si por alguna razon el kernel retorna a este punto. */

	jmp task_finished

task_finished:
	jmp task_finished

/* Rutina: kernel_exit
* Descripcion: Esta rutina lanza una interrupcion para invocar al kernel.
* Entrada:
*	Ninguno. Se supone que los parametros a enviar al kernel ya se encuentran
*			 en el tope de la pila de la tarea.
* Salida:
	Ninguno.
*/

kernel_exit:
	int $0x80

/*
 Seccion de datos de inicio de la tarea.
* Esta seccion proporciona los datos para el entorno de ejecucion de la tarea.
* Su proposito es almacenar los datos que se pasan como parametro a la rutina
* main() de la tarea.
* En el archivo link_module.ld se realiza la configuracion necesaria para que
* esta seccion de datos sea la primera que se almacena en el segmento
* de datos del archivo ejecutable. Luego de esta seccion se almacenan
* las variables globales definidas en el archivo .c de la tarea.
* En la rutina create_task (ver task.c) se almacenan los parametros en la
* pila de la tarea.
*/
.section .bootdata
task_arguments:
.space 512

/* 512 bytes de espacio para almacenar los datos que se pasan como parametro
* a la rutina main() de la tarea. Suficiente para almacenar 128 parametros
de tipo long (4 bytes). */
