/*
 * generic_list.h
 * Este archivo contiene el macro para definir una lista de un tipo especifico,
 * con el fin de ser instanciado con diferentes tipos de datos.
 */

#ifndef GENERIC_LIST_H_
#define GENERIC_LIST_H_

#define DEFINE_GENERIC_LIST_LINKS \
	void * next; /* Referencia al siguiente elemento */ 					   \
	void * prev; /* Referencia al anterior elemento */


#define DEFINE_GENERIC_LIST_TYPE(element_type)                                 \
typedef struct typelist_##element_type {                                       \
       element_type *head;                                                     \
       element_type *tail;                                                     \
       int count;                                                              \
}list_##element_type;                                                          \
                                                                               \
static __inline__ void init_list_##element_type(list_##element_type * l) {     \
     l->head = 0;						                                       \
     l->tail = 0;						                                       \
     l->count = 0;						                                       \
}                                                                              \
                                                                               \
static __inline__ element_type *                                               \
       push_front_##element_type(list_##element_type * l,                      \
                           element_type *  n) {                                \
       if (l ==0) return 0;                                                    \
                                                                               \
       n->next = 0;                                                            \
       n->prev = 0;                                                            \
       if (l->head == 0) { /*Primer elemento en la lista  */                   \
          l->head = n;                                          			   \
          l->tail = n;                                          		       \
          l->count = 0;                                                        \
       }else {                                                                 \
          n->next = l->head;                                   				   \
          l->head->prev = n;                                    			   \
          l->head = n;                                          			   \
       }                                                                       \
                                                                               \
       l->count++;                                                             \
                                                                               \
       return n;                                                               \
}                                                                              \
                                                                               \
static __inline__ element_type *                                               \
       push_back_##element_type(list_##element_type * l,                       \
                          element_type * n) {                                  \
       if (l ==0) return 0;                                                    \
       n->next = 0;                                                            \
       n->prev = 0;                                                            \
                                                                               \
       if (l->tail == 0) { /*Primer elemento en la lista */                    \
          l->head = n;                                                         \
          l->tail = n;                                                         \
          l->count = 0;                                                        \
       }else {                                                                 \
          n->prev = l->tail;                                                   \
          l->tail->next = n;                                                   \
          l->tail = n;                                                         \
       }                                                                       \
       l->count++;                                                             \
                                                                               \
       return n;                                                               \
}                                                                              \
                                                                               \
static __inline__ element_type * insert_ordered_##element_type(                \
                               list_##element_type * l,                        \
                          element_type * n) {                                  \
                                                                               \
    element_type * aux, *ant;                                                  \
    if (l==0) { return 0;}                                                     \
                                                                               \
    n->next = 0;                                                               \
    n->prev = 0;                                                               \
    if (l->head == 0) { /*Primer elemento en la lista  */                      \
          l->head = n;                                                         \
          l->tail = n;                                                         \
          l->count = 0;                                                        \
    }else {                                                                    \
		  /* Primero verificar en el extremo de la lista */					   \
		  if (compare_##element_type(l->tail, n) > 0) {						   \
			  ant = l->tail;												   \
			  aux = 0;														   \
		  }else {															   \
			  ant = 0;                                                         \
			  aux = l->head;                                                   \
			  while (aux != 0 && compare_##element_type(aux, n) > 0) {         \
				  ant = aux;                                                   \
				  aux = aux->next;                                             \
			  }                                                                \
		  }																	   \
                                                                               \
          n->prev = ant;                        				   			   \
                                                                               \
          if (ant == 0) { /*Primer nodo de la lista  */                        \
             n->next = l->head;                               				   \
             l->head->prev = n;                                 			   \
             l->head = n;                                                      \
          }else {                                                              \
              ant->next = n;                                    			   \
              n->next = aux;                                    			   \
              if (aux == 0) { /* Final de la lista */                          \
                 l->tail = n;                                                  \
              }else {                                                          \
                    aux->prev = n;                              			   \
              }                                                                \
          }                                                                    \
                                                                               \
    }                                                                          \
    l->count++;                                                                \
    return n;                                                                  \
}                                                                              \
static __inline__ element_type *                                             \
      remove_##element_type(list_##element_type * l,                          \
                                element_type * n ) {                           \
       element_type *ant, *aux;                                                \
                                                                               \
       if (l ==0) return 0;                                                    \
                                                                               \
       if (n == 0) return 0;                                                   \
                                                                               \
       ant = n->prev;                                        				   \
       aux = n->next;                                       				   \
                                                                               \
       /* Desconectar el nodo  de sus vecinos */                               \
       n->prev = 0;                                                            \
       n->next = 0;                                                            \
                                                                               \
       /* Conectar los nodos anterior y siguiente al nodo a eliminar*/         \
       if (ant != 0) {                                                         \
               ant->next = aux;                                 			   \
       }                                                                       \
                                                                               \
       if (aux != 0) {                                                         \
               aux->prev = ant;                                 			   \
       }                                                                       \
                                                                               \
       /* El nodo a remover era la cabeza? */                                  \
       if (n == l->head) {                                                     \
             l->head = aux; /* El siguiente es la nueva cabeza */              \
       }                                                                       \
                                                                               \
       /* El nodo a remover era la cola? */                                    \
       if (n == l->tail) {                                                     \
             l->tail = ant; /* El anterior es la nueva cola */                 \
       }                                                                       \
                                                                               \
        /* Un solo elemento? */                                                \
       if (l->tail == 0 || l->head == 0) { /* Un solo elemento? */             \
          l->head = 0;                                                         \
          l->tail = 0;                                                         \
       }                                                                       \
                                                                               \
       l->count--;                                                             \
                                                                               \
       return n;                                                               \
}                                                                              \
                                                                               \
static __inline__ element_type *                                               \
       front_##element_type(list_##element_type * l) {                         \
       if (l == 0) {return 0;}                                                 \
       if (l->head == 0) {return 0;}                                           \
       return l->head;                                                         \
}                                                                              \
                                                                               \
static __inline__ element_type *                                               \
       back_##element_type(list_##element_type * l) {                          \
       if (l ==0) return 0;                                                    \
             return l->tail;                                                   \
}                                                                              \
                                                                               \
static __inline__ element_type *                                               \
       pop_front_##element_type(list_##element_type * l) {                     \
       element_type *ret;                                                      \
                                                                               \
       if (l ==0) return 0;                                                    \
       /*printf("Count of nodes before pop_front: %d\n", l->count); */         \
       ret = l->head;                                                          \
       if (l->head == 0) {return ret;}                                         \
                                                                               \
       l->head = l->head->next;                              \
                                                                               \
       if (l->head == 0) {                                                     \
          l->tail = 0;                                                         \
       }else {                                                                 \
          l->head->prev = 0;                                                   \
       }                                                                       \
       l->count--;                                                             \
       /* printf("Count of nodes after pop_front: %d\n", l->count);    */      \
                                                                               \
       return ret;                                                             \
}                                                                              \
                                                                               \
static __inline__ element_type *                                               \
       pop_back_##element_type(list_##element_type * l) {                      \
       element_type *ret;                                                      \
                                                                               \
       if (l ==0) return 0;                                                    \
       ret = l->tail;                                                          \
                                                                               \
       if (l->tail == 0) {return ret;}                                         \
                                                                               \
       l->tail = l->tail->prev;                                \
                                                                               \
       if (l->tail == 0) { /* Un solo elemento? */                             \
          l->head = 0;                                                         \
       }else {                                                                 \
          l->tail->next = 0;                                                   \
       }                                                                       \
                                                                               \
       l->count--;                                                             \
                                                                               \
       return ret;                                                             \
}
#endif /* GENERIC_LIST_H_ */
